<?php
/**
 * Test fetching the free/busy information from the cache.
 *
 * PHP version 5
 *
 * @category   Kolab
 * @package    Kolab_FreeBusy
 * @subpackage UnitTests
 * @author     Gunnar Wrobel <wrobel@pardus.de>
 * @license    http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link       http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */

/**
 * Prepare the test setup.
 */
require_once dirname(__FILE__) . '/../Autoload.php';

/**
 * Test combining partial free/busy scripts into a single vCalendar.
 *
 * Copyright 2010 Klarälvdalens Datakonsult AB
 *
 * See the enclosed file COPYING for license information (LGPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
 *
 * @category   Kolab
 * @package    Kolab_FreeBusy
 * @subpackage UnitTests
 * @author     Gunnar Wrobel <wrobel@pardus.de>
 * @license    http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link       http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */
class Horde_Kolab_FreeBusy_Integration_CacheTest
extends PHPUnit_Extensions_Story_TestCase
{
    /**
     * The temporary directory used for caching.
     *
     * @param string;
     */
    private $cache_dir;

    /**
     * @scenario
     */
    public function testStoreCachesData()
    {
        $this->given('a temporary cache setup')
            ->and('a test owner of the cached data')
            ->when('storing partial data in the cache')
            ->then('the cache file will exist at location', 'example^org/test/test.pvc')
            ->and('the specified file cache will hold the specified content', 'example^org/test/test.pvc', 'TEST');
    }

    /**
     * @scenario
     */
    public function testDeleteRemovedCachedData()
    {
        $this->given('a temporary cache setup')
            ->and('a test owner of the cached data')
            ->when('storing partial data in the cache')
            ->and('deleting partial data from the cache')
            ->then('there will be no cache file at location', 'example^org/test/test.pvc');
    }

    /**
     * @scenario
     */
    public function testLoadLoadsCachedData()
    {
        $this->given('a temporary cache setup')
            ->and('a test owner of the cached data')
            ->when('storing partial data in the cache')
            ->then('loading the extended cached data will return', 'TEST');
    }

    /**
     * @scenario
     */
    public function testStoreCachesDataWhenUsingAcl()
    {
        $this->given('a temporary cache setup')
            ->and('using ACL')
            ->and('a test owner of the cached data')
            ->when('storing partial data in the cache')
            ->then('the cache file will exist at location', 'example^org/test/test.pvc')
            ->and('the specified file cache will hold the specified content', 'example^org/test/test.pvc', 'TEST');
    }

    /**
     * @scenario
     */
    public function testDeleteRemovedCachedDataWhenUsingAcl()
    {
        $this->given('a temporary cache setup')
            ->and('using ACL')
            ->and('a test owner of the cached data')
            ->when('storing partial data in the cache')
            ->and('deleting partial data from the cache')
            ->then('there will be no cache file at location', 'example^org/test/test.pvc');
    }

    /**
     * @scenario
     */
    public function testLoadLoadsCachedDataWhenUsingAcl()
    {
        $this->given('a temporary cache setup')
            ->and('using ACL')
            ->and('a test owner of the cached data')
            ->when('storing partial data in the cache')
            ->then('loading the extended cached data will return', 'TEST');
    }

    /**
     * @scenario
     */
    public function testDeleteownerRemovesOwnerData()
    {
        $this->given('a temporary cache setup')
            ->and('using ACL')
            ->and('a test owner of the cached data')
            ->when('storing partial data in the cache')
            ->and('deleting the cache data for the test owner')
            ->then('there will be no cache file at location', 'example^org/test/test.pvc')
            ->and('there will be no data in the databases for the test user.');
    }

    /**
     * Handle a "given" step.
     *
     * @param array  &$world    Joined "world" of variables.
     * @param string $action    The description of the step.
     * @param array  $arguments Additional arguments to the step.
     *
     * @return mixed The outcome of the step.
     */
    public function runGiven(&$world, $action, $arguments)
    {
        switch($action) {
        case 'a temporary cache setup':
            //todo H4 Horde::getTempDir
            $this->cache_dir = Horde::getTempDir() . '/Kolab_FreeBusy_Test_' . md5(uniqid(mt_rand(), true));
            mkdir($this->cache_dir);
            $world['cache_structure'] = new Horde_Kolab_FreeBusy_Cache_Structure_Base(
                $this->cache_dir
            );
            break;
        case 'using ACL':
            global $conf;
            $conf['fb']['use_acls'] = true;
            break;
        case 'a test owner of the cached data':
            $world['owner'] = new Horde_Kolab_FreeBusy_Stub_Owner();
            $world['cache'] = new Horde_Kolab_FreeBusy_Cache(
                $world['cache_structure'], $world['owner'] 
            );
            break;
        default:
            return $this->notImplemented($action);
        }
    }

    /**
     * Handle a "when" step.
     *
     * @param array  &$world    Joined "world" of variables.
     * @param string $action    The description of the step.
     * @param array  $arguments Additional arguments to the step.
     *
     * @return mixed The outcome of the step.
     */
    public function runWhen(&$world, $action, $arguments)
    {
        switch($action) {
        case 'storing partial data in the cache':
            $request = new Horde_Kolab_FreeBusy_Stub_Request();
            $request->setFolder('INBOX/test');
            $world['cache']->storePartial(
                new Horde_Kolab_FreeBusy_Stub_User(),
                new Horde_Kolab_FreeBusy_Params_Freebusy_Folder(
                    $request
                ),
                new Horde_Kolab_FreeBusy_Stub_Resource(),
                'TEST'
            );
            break;
        case 'deleting partial data from the cache':
            $request = new Horde_Kolab_FreeBusy_Stub_Request();
            $request->setFolder('INBOX/test');
            $world['cache']->deletePartial(
                new Horde_Kolab_FreeBusy_Params_Freebusy_Folder(
                    $request
                )
            );
            break;
        case 'deleting the cache data for the test owner':
            $world['cache']->deleteOwner();
            break;
        default:
            return $this->notImplemented($action);
        }
    }

    /**
     * Handle a "then" step.
     *
     * @param array  &$world    Joined "world" of variables.
     * @param string $action    The description of the step.
     * @param array  $arguments Additional arguments to the step.
     *
     * @return mixed The outcome of the step.
     */
    public function runThen(&$world, $action, $arguments)
    {
        switch($action) {
        case 'the cache file will exist at location':
            $this->assertTrue(
                file_exists(
                    $this->cache_dir . DIRECTORY_SEPARATOR . $arguments[0]
                )
            );
            break;
        case 'there will be no cache file at location':
            $this->assertFalse(
                file_exists(
                    $this->cache_dir . DIRECTORY_SEPARATOR . $arguments[0]
                )
            );
            break;
        case 'the specified file cache will hold the specified content':
            $cached = unserialize(
                file_get_contents(
                    $this->cache_dir . DIRECTORY_SEPARATOR . $arguments[0]
                )
            );
            $this->assertEquals($arguments[1], $cached['data']);
            break;
        case 'loading the extended cached data will return':
            $request = new Horde_Kolab_FreeBusy_Stub_Request();
            $request->setFolder('INBOX/test');
            $result = $world['cache']->loadPartial(
                new Horde_Kolab_FreeBusy_Stub_User(),
                new Horde_Kolab_FreeBusy_Params_Freebusy_Folder(
                    $request
                ),
                true
            );
            $this->assertEquals($arguments[0], $result);
            break;
        case 'there will be no data in the databases for the test user.':
            $this->assertEquals(
                array(),
                $world['cache_structure']->getAclDbCache()->get('test@example.org')
            );
            $this->assertEquals(
                array(),
                $world['cache_structure']->getXaclDbCache()->get('test@example.org')
            );
            break;
        default:
            return $this->notImplemented($action);
        }
    }

    public function setUp()
    {
    }

    public function tearDown()
    {
        foreach (new RecursiveIteratorIterator(new RecursiveDirectoryIterator($this->cache_dir), RecursiveIteratorIterator::CHILD_FIRST) as $file) {
            if ($file->isFile()) {
                unlink($file->getPathname());
            }
        }
        foreach (new RecursiveIteratorIterator(new RecursiveDirectoryIterator($this->cache_dir), RecursiveIteratorIterator::CHILD_FIRST) as $file) {
            if ($file->isDir()) {
                rmdir($file->getPathname());
            }
        }
        rmdir($this->cache_dir);
    }
}